#!/bin/sh
#@(#)start_lmgrd.sh version: 1.2  Date: 8/22/2008 $
#@(#)(C) Copyright IBM Corporation. 2008. All Rights Reserved.
#-------------------------------------------------------------------------
# These 4 LICENSE variables need to be modified to match your installation:
LICENSE_USER=joe                   # root will start lmgrd by this user name
                                   # This user must be in the /etc/passwd file.
LICENSE_DIR=/usr/flexlm            # FLEXlm directory with lmgrd, rational, etc.
                                   # This is where the utilites are installed.
LICENSE_DAT=$LICENSE_DIR/pluto.dat # FLEXlm license file.  Needs to be the
                                   # full path name to your license file.
LICENSE_LOG=$LICENSE_DIR/pluto.log # FLEXlm lmgrd log.  This is where you can
                                   # find valuable lmgrd messages. 
#-------------------------------------------------------------------------
# The next 8 lines are for insserv on Linux SUSE
### BEGIN INIT INFO
# Provides:       ibmrational
# Required-Start: $network
# Required-Stop:
# Default-Start:  3 5
# Default-Stop:
# Description:    Start the IBM Rational license server
### END INIT INFO
#SOHT
# Start and stop the license manager for IBM Rational
#
# This script can be called with the following command line options:
#
#   help       Show this usage information.
#
#   start      This will start the lmgrd license server processes.
#              This is the default action if no command line argument
#              is specified.  The rational vendor daemon will start
#              if there are any rational licenses in the license file.
#              The ibmratl vendor daemon will start if there are any
#              ibmratl licenses in the license file.
#
#   stop       Stop the license server processes.
#
#   lmstat     Run the lmstat command. You can do specify other options
#              to lmstat.  For example, 'lmstat -a' or 'lmstat -f feature'.
#
#   lmreread   Run the lmreread command.  This tells the license server to
#              reread the license file.
#
#   exinstal   Run the exinstal command on the license file to check the
#              validity of the $vendor_daemon license codes.
#
#   exinstrl   Run the exinstrl command on the license file to check the
#              validity of the $trl_daemon license codes.
#
#   lmdiag     Run the lmdiag command on the license file.
#
#   ishow      Same as the install option, except DO NOT install.
#              This option shows the commands to do the install.
#              Use the install option to do the install.
#        
#   install    Once this script is working, you will want to copy this
#              script to the location where the server can run it when
#              it reboots.  Use this option to install this start script.
#
#              To see what the install will do, use the ishow option.
#
#              You must be root on the license server machine to install.
# 
#   monitor    Monitor the license server.  You can also do this by
#              running the UNIX tail command like this:
#                 tail -f LICENSE_LOG
#
#   start_msg  These actions are for the system rc scripts use.
#   stop_msg   They display a message.
#EOHT
#-------------------------------------------------------------------------

script=$0
instCmd=showCmd

showUsage() {
    if [ -n "$1" ]
    then
        cmd=`basename $script`
        echo "$cmd: $1"
        echo "Run '$script help' to get usage information."
        exit 1
    else
        sed -e '1,/^#SOHT/d' -e '/^#EOHT/,$d' -e 's/^#//' \
            -e "s?LICENSE_LOG?$LICENSE_LOG?" \
            $script | more
        exit 0
    fi
}

showCmd() {
    if [ "$1" = "rm" ]
    then
        shift
        for _file in $*
        do
            test -f $_file || continue
            echo "  rm -f $_file"
        done
    else
        echo "  $*"
    fi
}

runCmd() {
    if [ "$1" = "rm" ]
    then
        shift
        for _file in $*
        do
            test -f $_file || continue
            echo "rm -f $_file"
            rm -f $_file
        done
    else
        echo "$*"
        eval $*
    fi
}

setServerList() {
    serverList="lmgrd"
    for _server in rational ibmratl
    do
        if [ -x $LICENSE_DIR/$_server ]
        then
            serverList="$serverList/$_server"
        fi
    done
}

case "$1" in
help|-h*|-?|\? )
    showUsage ""
    ;;

stop_msg )
    setServerList
    echo "Stop $serverList license servers"
    exit 0
    ;;

stop )
    # Stop license manager.
    setServerList
    echo "Stop $serverList license servers"
    echo "Shutting down lmgrd/$serverList license servers..."
    $LICENSE_DIR/lmdown -q -c $LICENSE_DAT
    exit $?
    ;;

monitor )
    if [ -s $LICENSE_LOG ]
    then
        echo "tail -f $LICENSE_LOG"
        exec tail -f $LICENSE_LOG
        exit $?
    fi
    LOG_STATUS="The lmgrd log file ($LICENSE_LOG)"
    if [ -f $LICENSE_LOG ]
    then
        echo "$LOG_STATUS is empty."
    else
        echo "$LOG_STATUS does not exist."
    fi
    exit 1
    ;;

lmreread|reread )
    # Tell license manager to reread the license file.
    echo "$LICENSE_DIR/lmreread -c $LICENSE_DAT"
    $LICENSE_DIR/lmreread -c $LICENSE_DAT
    exit $?
    ;;

lmdiag|diag )

    # Do lmdiag command. Pass any other parameters to the command.
    shift
    echo "$LICENSE_DIR/lmdiag -c $LICENSE_DAT -n $*"
    $LICENSE_DIR/lmdiag -c $LICENSE_DAT -n $*
    exit $?
    ;;

lmstat|stat )
    # Do lmstat command. Pass any other parameters to the command.
    shift
    echo "$LICENSE_DIR/lmstat -c $LICENSE_DAT $*"
    $LICENSE_DIR/lmstat -c $LICENSE_DAT $*
    exit $?
    ;;

exinstal|ex )
    # Check the 'rational' licenses using exinstal.
    echo "$LICENSE_DIR/exinstal $LICENSE_DAT"
    $LICENSE_DIR/exinstal $LICENSE_DAT
    exit $?
    ;;

exinstrl|trl )
    # Check the 'ibmratl' licenses using exinstrl.
    echo "$LICENSE_DIR/exinstrl $LICENSE_DAT"
    $LICENSE_DIR/exinstrl $LICENSE_DAT
    exit $?
    ;;

start_msg )
    setServerList
    echo "Start $serverList license servers"
    exit 0
    ;;

ishow|install )
    # Set up lmgrd start script to run when this machine reboots.
    OS=`uname -s`
    if [ "$1" = install ]
    then
        if id | grep "uid=0" >/dev/null 2>&1
        then
            instCmd=runCmd
        else
            showUsage "To $1, you must run this script as root."
        fi
    else
        echo "# Commands to install $script on `uname -n`:"
    fi
    case $OS in
    AIX )
        # On AIX, edit /etc/inittab. Add this line:
        if grep "^lmgr:" /etc/inittab
        then
            echo "lmgr already configured in the /etc/inittab file."
        else
            if [ "$instCmd" != "runCmd" ]
            then
                echo "  echo 'lmgr:234:once:/bin/sh $script' >> /etc/inittab"
            else
                echo "lmgr:234:once:/bin/sh $script" >> /etc/inittab
                grep "^lmgr:" /etc/inittab
                if grep "^lmgr:" /etc/inittab
                then
                    echo "lmgr added to the /etc/inittab file."
                else
                    echo "Failed to add lmgr to the /etc/inittab file."
                    exit 1
                fi
            fi
        fi
        exit 0
        ;;
    HP-UX )
        # On HP-UX
        $instCmd rm /sbin/init.d/rational /sbin/rc2.d/[KS]98Rational 
        $instCmd cp $script /sbin/init.d/rational
        $instCmd ln -s /sbin/init.d/rational /sbin/rc2.d/S98Rational
        $instCmd ln -s /sbin/init.d/rational /sbin/rc2.d/K98Rational
        exit $?
        ;;
    Linux )
        if [ -f /sbin/yast2 -a -x /sbin/chkconfig ]
        then
            # On Linux with YAsT and chkconfig, install like this.
            $instCmd rm /etc/init.d/ibmrational
            $instCmd cp $script /etc/init.d/ibmrational
            $instCmd /sbin/chkconfig --add ibmrational
            $instCmd /sbin/chkconfig --level 35 ibmrational on
            $instCmd /sbin/chkconfig --list | grep ibmrational
            $instCmd /sbin/insserv /etc/init.d/ibmrational
            exit $?
        else
            # On other Linux
            $instCmd rm /etc/init.d/rational /etc/rc[35].d/[KS]98Rational
            $instCmd cp $script /etc/init.d/rational
            $instCmd ln -s ../init.d/rational /etc/rc3.d/S98Rational
            $instCmd ln -s ../init.d/rational /etc/rc3.d/K98Rational
            $instCmd ln -s ../init.d/rational /etc/rc5.d/S98Rational
            $instCmd ln -s ../init.d/rational /etc/rc5.d/K98Rational
            exit $?
        fi
        ;;
    SunOS )
        # On Solaris
        $instCmd rm /etc/init.d/rational /etc/rc[35].d/[KS]98Rational
        $instCmd cp $script /etc/init.d/rational
        $instCmd ln -s ../init.d/rational /etc/rc2.d/S98Rational
        $instCmd ln -s ../init.d/rational /etc/rc2.d/K98Rational
        exit $?
        ;;
    * )
        showUsage "Can only run on AIX, Linux, SunOS, or HP-UX."
        ;;
    esac
    ;;

""|start|Start|START )
    # Fall through to start server
    ;;
* )
    showUsage "Command option '$1' not recognized."
    ;;
esac

# Start server if we can.
LICENSE_EXE=$LICENSE_DIR/lmgrd
if [ ! -x $LICENSE_EXE ]
then
    echo "IBM Rational license manager not found:"
    echo "  $LICENSE_EXE"
    exit 1
fi

if [ ! -f $LICENSE_DAT ]
then
    echo "License data file not found:"
    echo "  $LICENSE_DAT"
    exit 1
fi

# Start lmgrd/rational/ibmratl license servers...
#
# If this script is run as root and LICENSE_USER is not root
# use 'su LICENSE_USER' to run it as that user.
doSuCmd=no
if id | grep "uid=0" >/dev/null 2>&1
then
    # Script run by root--use su command if LICENSE_USER is not root.
    if [ "$LICENSE_USER" != root ]
    then
        doSuCmd=yes
    fi
fi

if [ -s $LICENSE_LOG ]
then
    rmPri="/bin/rm -f $LICENSE_LOG.prior"
    mvLog="/bin/mv $LICENSE_LOG  $LICENSE_LOG.prior"
    if [ "$doSuCmd" = yes ]
    then
        su $LICENSE_USER -c "$rmPri ; $mvLog"
    else
        eval $rmPri
        eval $mvLog
    fi
fi

lowerFdLimit=no
case `uname -s` in
SunOS )
    # Check the hard limit for the number of file descriptors.
    # Needs to be 1024 according to Globetrotter.
    actualFdLimit=`ulimit -H -n`
    case "$actualFdLimit" in
    [0-9]* )
        if [ "$actualFdLimit" -gt 1024 ]
        then
            lowerFdLimit=yes
        fi
        ;;
    unlimited )
        lowerFdLimit=yes
        ;;
    esac
    ;;
esac

if [ "$lowerFdLimit" = yes ]
then
    # Lower the file descriptor limits.
    # This is done on Solaris to workaround a problem with the
    # select() system call in FLEXlm and this limit above 1024.
    ulimit -n 1024
    ulimit -H -n 1024
fi

setServerList
START_LMGRD="$LICENSE_EXE -c $LICENSE_DAT -l $LICENSE_LOG"
if [ "$doSuCmd" = yes ]
then
    echo "Starting FLEXlm $serverList license servers as $LICENSE_USER."
    su $LICENSE_USER -c "$START_LMGRD &" >/dev/null 2>&1
else
    echo "Starting FLEXlm $serverList license servers."
    $START_LMGRD 2>$LICENSE_LOG &
fi

# End of start script.
